/*********************************************************************
 *       This file is part of the Mandala library source code.       *
 *                   Copyright (C) 2002                              *
 *                                                                   *
 *       Authors: eipi - eipiequalsnoone@userOps.sf.net               *
 *                                                                   *
 * This library is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU Lesser General Public        *
 * License as published by the Free Software Foundation; either      *
 * version 2.1 of the License, or (at your option) any later version.*
 *                                                                   *
 * This library is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *
 * Lesser General Public License for more details.                   *
 *                                                                   *
 * You should have received a copy of the GNU Lesser General Public  *
 * License along with this library; if not, write to                 *
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   *
 * Boston, MA  02111-1307  USA                                       *
 *********************************************************************/
package org.ow2.novabpm.util;

import java.util.logging.LogRecord;
import java.util.logging.SimpleFormatter;

/**
 * <p>
 * Custom implementation of the {@link SimpleFormatter} class.
 * <p>
 * 
 * <p>
 * This implementation produce the following pattern:<br>
 * <center><code>LEVEL DELAY THREAD Class.Method(): MSG</code></center>
 * 
 * @author <a href="mailto:eipiequalsnoone@userOps.sf.net">eipi</a>
 * @version $Revision: 1.4 $
 * @since 1.0
 */
public class DetailedFormatter extends SimpleFormatter {
	final long startChrono = System.currentTimeMillis();

	@Override
    public String format(final LogRecord record) {
    	final StringBuffer sb = new StringBuffer(160); // Two 80 colons lines.
    	
    	sb.append(record.getLevel());
    	sb.append(' ');
    	final long delay = record.getMillis() - startChrono;
    	sb.append(Misc.format(delay));
    	sb.append(' ');
    	sb.append(record.getThreadID());
    	sb.append(' ');
    	sb.append(record.getSourceClassName());
    	sb.append('.');
    	sb.append(record.getSourceMethodName());
    	sb.append('(');
    	final String msg = record.getMessage();
    	if (msg.startsWith("ENTRY")) {
    		final Object[] params = record.getParameters();
    		if (params != null) {
    			final int last = params.length - 1;
    			for (int i = 0; i < last; i++) {
    				sb.append(params[i]);
    				sb.append(", ");
    			}
    			sb.append(params[last]);
    		}
    	}
    	sb.append(')');
    	if (msg.startsWith("RETURN")) {
    		sb.append(" -> ");
    		sb.append(record.getParameters()[0]);
    	}
    	sb.append(Misc.LINE_SEPARATOR);
    	
    	return new String(sb);
    }
}
